@@ -5,7 +5,7 @@ use warnings;
 package MooX::Press;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.006';
+our $VERSION   = '0.007';
 
 use Exporter::Tiny qw(mkopt);
 use Scalar::Util qw(blessed);
@@ -495,13 +495,29 @@ sub _make_package {
 	return $qname;
 }
 
+my %_cached_moo_helper;
+sub _get_moo_helper {
+	my $builder = shift;
+	my ($package, $helpername) = @_;
+	return $_cached_moo_helper{"$package\::$helpername"}
+		if $_cached_moo_helper{"$package\::$helpername"};
+	die unless $helpername =~ /^(has|with|extends|around|before|after)$/;
+	my $tracker = ($INC{'Moo/Role.pm'} && 'Moo::Role'->is_role($package))
+		? $Moo::Role::INFO{$package}{exports}
+		: $Moo::MAKERS{$package}{exports};
+	if (ref $tracker) {
+		return ($_cached_moo_helper{"$package\::$helpername"} = $tracker->{$helpername});
+	}
+	# I hate this...
+	$_cached_moo_helper{"$package\::$helpername"} =
+		eval sprintf('do { package %s; use Moo; my $coderef = \&%s; no Moo; $coderef };', $package, $helpername);
+}
+
 sub make_attribute_moo {
 	my $builder = shift;
 	my ($class, $attribute, $spec) = @_;
-	my $tracker = ($INC{'Moo/Role.pm'} && 'Moo::Role'->is_role($class))
-		? $Moo::Role::INFO{$class}{exports}
-		: $Moo::MAKERS{$class}{exports};
-	$tracker->{has}->($attribute, %$spec);
+	my $helper = $builder->_get_moo_helper($class, 'has');
+	$helper->($attribute, %$spec);
 }
 
 sub make_attribute_moose {
@@ -521,8 +537,8 @@ sub make_attribute_mouse {
 sub extend_class_moo {
 	my $builder = shift;
 	my ($class, $isa) = @_;
-	my $tracker = $Moo::MAKERS{$class}{exports};
-	$tracker->{extends}->(@$isa);
+	my $helper = $builder->_get_moo_helper($class, 'extends');
+	$helper->(@$isa);
 }
 
 sub extend_class_moose {
@@ -542,10 +558,8 @@ sub extend_class_mouse {
 sub apply_roles_moo {
 	my $builder = shift;
 	my ($class, $roles) = @_;
-	my $tracker = ($INC{'Moo/Role.pm'} && 'Moo::Role'->is_role($class))
-		? $Moo::Role::INFO{$class}{exports}
-		: $Moo::MAKERS{$class}{exports};
-	$tracker->{with}->(@$roles);
+	my $helper = $builder->_get_moo_helper($class, 'with');
+	$helper->(@$roles);
 }
 
 sub apply_roles_moose {
@@ -596,14 +610,8 @@ sub install_constants {
 sub modify_method_moo {
 	my $builder = shift;
 	my ($class, $modifier, $method_name, $coderef) = @_;
-	if ($INC{'Moo/Role.pm'} && 'Moo::Role'->is_role($class)) {
-		push @{$Moo::Role::INFO{$class}{modifiers}||=[]}, [ $modifier, $method_name, $coderef ];
-		'Moo::Role'->_maybe_reset_handlemoose($class);
-	}
-	else {
-		require Class::Method::Modifiers;
-		Class::Method::Modifiers::install_modifier(@_);
-	}
+	my $helper = $builder->_get_moo_helper($class, $modifier);
+	$helper->($method_name, $coderef);
 }
 
 sub modify_method_moose {
